'use strict';

Object.defineProperty(exports, '__esModule', { value: true });

var react = require('react');

const createComponent = (name, base, modifiers, properties) => {
    const componentCache = new Map();
    function createReactCCMComponent(tag) {
        const Component = react.forwardRef((props, ref) => {
            const style = {};
            // Start off with our base component className,
            // and the className if any passed in to us
            let className = `${props.className || ""} ${base} `;
            const remainingProps = {};
            for (const [propName, propValue] of Object.entries(props)) {
                // First process the class modifiers that can use. Add them to our list of classNames
                // if the prop value is truthy
                if (propName in modifiers && propValue) {
                    className += " " + modifiers[propName];
                }
                // Next if the prop is a custom property passed in, we need to set the value on style
                else if (propName in properties) {
                    style[properties[propName]] = propValue;
                }
                // Assume the prop was intended for the underlying element and keep it as-is
                else {
                    remainingProps[propName] = propValue;
                }
            }
            /// Create the react element, combining all the props through
            return react.createElement(tag, Object.assign(Object.assign({}, remainingProps), { ref, className: className.trim(), style }));
        });
        // Make sure we have a pretty name that is visible in the debugger
        Component.displayName = `CCM-${name}.${typeof tag === "string" ? tag : tag.displayName || tag.name}`;
        return Component;
    }
    const lookupComponent = (tag) => {
        // .extend() is actually just a fancy way of doing prop lookup
        if (tag === "extend") {
            return lookupComponent;
        }
        if (!componentCache.has(tag)) {
            componentCache.set(tag, createReactCCMComponent(tag));
        }
        return componentCache.get(tag);
    };
    return new Proxy(Object.create(null), {
        get(unused, tag) {
            if (typeof tag === "symbol")
                return;
            return lookupComponent(tag);
        },
    });
};

exports.createComponent = createComponent;
