/// <amd-module name="@css-components/react" />
import { ComponentType } from "react";
export declare type ReactCCMProps<Modifiers extends string, Properties extends string> = Partial<Record<Modifiers, boolean>> & Record<Properties, string>;
export declare type DomComponentType<CCMProps> = {
    [ElementName in keyof JSX.IntrinsicElements]: ComponentType<JSX.IntrinsicElements[ElementName] & CCMProps>;
};
export interface ExtendableComponentType<CCMProps> {
    /**
     * Escape Hatch: Do not use regularly!
     *
     * Manually call .extend('div') with the name of JSX DOM element
     * to support dynamic extension
     */
    extend<ElementName extends keyof JSX.IntrinsicElements>(elementName: ElementName): ComponentType<JSX.IntrinsicElements[ElementName] & CCMProps>;
    /**
     * Escape Hatch: Do not use regularly!
     *
     * Manually call .extend(OtherComponent) with another component
     * to support dynamic extension
     */
    extend<OtherComponentProps>(OtherComponent: ComponentType<OtherComponentProps>): ComponentType<OtherComponentProps & CCMProps>;
}
/**
 * Given CCM Props for a component, the fully accessible component available to be used
 */
export declare type CCMComponentType<Modifiers extends string, Properties extends string> = ExtendableComponentType<ReactCCMProps<Modifiers, Properties>> & DomComponentType<ReactCCMProps<Modifiers, Properties>>;
export declare const createComponent: <Modifiers extends string, Properties extends string>(name: string, base: string, modifiers: Record<Modifiers, string>, properties: Record<Properties, string>) => CCMComponentType<Modifiers, Properties>;
